@@ -42,6 +42,7 @@
 
 #include <iomanip>
 #include <iostream>
+#include <algorithm>
 
 namespace muSpectre {
 
@@ -49,28 +50,83 @@ namespace muSpectre {
   Eigen::IOFormat format(Eigen::FullPrecision, 0, ", ", ",\n", "[", "]", "[",
                          "]");
 
+  /* ---------------------------------------------------------------------- */
+  ConvergenceCriterion::ConvergenceCriterion()
+      : was_last_step_linear_test{false}, equil_tol_test{false},
+        newton_tol_test{false} {}
+
+  /* ---------------------------------------------------------------------- */
+  bool & ConvergenceCriterion::get_was_last_step_linear_test() {
+    return this->was_last_step_linear_test;
+  }
+
+  /* ---------------------------------------------------------------------- */
+  bool & ConvergenceCriterion::get_equil_tol_test() {
+    return this->equil_tol_test;
+  }
+
+  /* ---------------------------------------------------------------------- */
+  bool & ConvergenceCriterion::get_newton_tol_test() {
+    return this->newton_tol_test;
+  }
+
+  /* ---------------------------------------------------------------------- */
+  void ConvergenceCriterion::reset() {
+    this->was_last_step_linear_test = false;
+    this->equil_tol_test = false;
+    this->newton_tol_test = false;
+  }
+
   //--------------------------------------------------------------------------//
   std::vector<OptimizeResult>
   newton_cg(Cell & cell, const LoadSteps_t & load_steps,
-            KrylovSolverBase & solver, Real newton_tol, Real equil_tol,
-            Verbosity verbose, IsStrainInitialised strain_init) {
+            KrylovSolverBase & solver, const Real & newton_tol,
+            const Real & equil_tol, const Verbosity & verbose,
+            const IsStrainInitialised & strain_init,
+            EigenStrainOptFunc_ref eigen_strain_func) {
+    if (load_steps.size() == 0) {
+      throw SolverError("No load steps specified.");
+    }
+
     const auto & comm = cell.get_communicator();
 
     using Matrix_t = Eigen::Matrix<Real, Eigen::Dynamic, Eigen::Dynamic>;
-
     auto shape{cell.get_strain_shape()};
 
+    ConvergenceCriterion convergence_criterion;
+
     // create a field collection to store workspaces
     auto field_collection{cell.get_fields().get_empty_clone()};
     // Corresponds to symbol δF or δε
     muGrid::MappedField<muGrid::FieldMap<Real, Mapping::Mut>> incrF_field{
-        "incrF",          shape[0], shape[1], IterUnit::SubPt,
-        field_collection, QuadPtTag};
+        "incrF",         shape[0],         shape[1],
+        IterUnit::SubPt, field_collection, QuadPtTag};
 
     // field to store the rhs for cg calculations
     muGrid::MappedField<muGrid::FieldMap<Real, Mapping::Mut>> rhs_field{
-        "rhs",    shape[0], shape[1], IterUnit::SubPt, field_collection,
-        QuadPtTag};
+        "rhs",           shape[0],         shape[1],
+        IterUnit::SubPt, field_collection, QuadPtTag};
+
+    // Standard strain field without the eigen strain: typically the strain
+    // field given to the cell for stress/tangent evaluation, but in case of
+    // presence of eigen strain, it is the strain field before adding the eigen
+    // strain
+    muGrid::TypedFieldBase<Real> & general_strain_field{
+        [&cell, &field_collection,
+         &eigen_strain_func]() -> muGrid::TypedFieldBase<Real> & {
+          if (not(eigen_strain_func == muGrid::nullopt)) {
+            // general strain field
+            muGrid::RealField & general_strain_field{
+                field_collection.register_real_field(
+                    "general strain",
+                    shape_for_formulation(cell.get_formulation(),
+                                          cell.get_material_dim()),
+                    QuadPtTag)};
+            return general_strain_field;
+          } else {
+            return cell.get_strain();
+          }
+        }()};
 
     solver.initialise();
 
@@ -182,8 +238,10 @@ namespace muSpectre {
     // storage for the previous mean strain (to compute ΔF or Δε )
     Matrix_t previous_macro_strain{load_steps.back().Zero(shape[0], shape[1])};
 
-    auto & F{cell.get_strain()};
-    //! incremental loop
+    // strain field used by the cell for evaluating stresses/tangents.
+    auto & eval_strain_field{cell.get_strain()};
+
+    //! incremental loop (load steps)
     for (const auto & tup : akantu::enumerate(load_steps)) {
       const auto & strain_step{std::get<0>(tup)};
       const auto & macro_strain{std::get<1>(tup)};
@@ -191,11 +249,13 @@ namespace muSpectre {
         std::cout << "at Load step " << std::setw(count_width)
                   << strain_step + 1 << std::endl;
       }
+
       // updating cell strain with the difference of the current and previous
       // strain input.
-      for (auto && strain : muGrid::FieldMap<Real, Mapping::Mut>(
-               F, shape[0], muGrid::IterUnit::SubPt)) {
-        strain += macro_strain - previous_macro_strain;
+      auto && F_general_map{muGrid::FieldMap<Real, Mapping::Mut>(
+          general_strain_field, shape[0], muGrid::IterUnit::SubPt)};
+      for (auto && strain_general : F_general_map) {
+        strain_general += macro_strain - previous_macro_strain;
       }
 
       // define the number of newton iteration (initially equal to 0)
@@ -203,43 +263,68 @@ namespace muSpectre {
       std::string message{"Has not converged"};
       Real incr_norm{2 * newton_tol}, grad_norm{1};
       Real stress_norm{2 * equil_tol};
+      bool has_converged{false};
 
       //! Checks two loop breaking criteria (newton tolerance and equilibrium
       //! tolerance)
-      auto convergence_test = [&incr_norm, &grad_norm, &newton_tol,
-                               &stress_norm, &equil_tol, &message]() {
-        bool incr_test = incr_norm / grad_norm <= newton_tol;
-        bool stress_test = stress_norm < equil_tol;
-        if (incr_test) {
-          message = "Residual  tolerance reached";
-        } else if (stress_test) {
-          message = "Reached stress divergence tolerance";
-        }
-        return incr_test || stress_test;
-      };
+      auto && early_convergence_test{
+          [&incr_norm, &grad_norm, &newton_tol, &stress_norm, &equil_tol,
+           &message, &has_converged, &convergence_criterion]() {
+            convergence_criterion.get_newton_tol_test() =
+                (incr_norm / grad_norm) <= newton_tol;
+            convergence_criterion.get_equil_tol_test() =
+                stress_norm < equil_tol;
 
-      //! carries out a single newton_cg solving step
-      auto iterate_solve = [&cell, &rhs_field, &incrF_field, &F, &stress_norm,
-                            &incr_norm, &grad_norm, &comm, &convergence_test,
-                            &solver, &strain_step, &load_steps, &macro_strain,
-                            &previous_macro_strain, &newt_iter, &strain_symb,
-                            &verbose, &count_width, &newton_tol, &shape]() {
-        // obtain material response
+            if (convergence_criterion.get_newton_tol_test()) {
+              message = "Residual  tolerance reached";
+            } else if (convergence_criterion.get_equil_tol_test()) {
+              message = "Reached stress divergence tolerance";
+            }
+            has_converged = convergence_criterion.get_newton_tol_test() or
+                            convergence_criterion.get_equil_tol_test();
+            return has_converged;
+          }};
+
+      //! Checks all convergence criteria, including detection of linear
+      //! problems
+      auto && full_convergence_test{[&early_convergence_test, &cell,
+                                     &has_converged, &message,
+                                     &convergence_criterion]() {
+        convergence_criterion.get_was_last_step_linear_test() =
+            not cell.was_last_eval_non_linear();
+        if (convergence_criterion.get_was_last_step_linear_test()) {
+          message = "Linear problem, no more iteration necessary";
+        }
+        has_converged = early_convergence_test() or
+                        convergence_criterion.get_was_last_step_linear_test();
+        return has_converged;
+      }};
+
+      // Iterative update for the non-linear case
+      for (; newt_iter < solver.get_maxiter() && !has_converged; ++newt_iter) {
+        // updating the strain fields with the  eigen_strain field calculated by
+        // the functor called eigen_strain_func
+
+        if (not(eigen_strain_func == muGrid::nullopt)) {
+          eval_strain_field = general_strain_field;
+          (eigen_strain_func.value())(strain_step, eval_strain_field);
+        }
         auto res_tup{cell.evaluate_stress_tangent()};
         auto & P{std::get<0>(res_tup)};
+
         auto & rhs{rhs_field.get_field()};
 
         rhs = -P;
         cell.apply_projection(rhs);
         stress_norm = std::sqrt(comm.sum(rhs.eigen_vec().squaredNorm()));
 
-        if (convergence_test()) {
-          return true;
+        if (early_convergence_test()) {
+          break;
         }
-        // calling solver for solving the current (iteratively approximated)
+
+        // calling sovler for solving the current (iteratively approximated)
         // linear equilibrium problem
         auto & incrF{incrF_field.get_field()};
-
         try {
           incrF = solver.solve(rhs.eigen_vec());
         } catch (ConvergenceError & error) {
@@ -259,12 +344,13 @@ namespace muSpectre {
 
         // updating cell strain with the periodic (non-constant) solution
         // resulted from imposing the new macro_strain
-        F += incrF;
+        general_strain_field += incrF;
 
         // updating the incremental differences for checking the termination
         // criteria
         incr_norm = std::sqrt(comm.sum(incrF.eigen_vec().squaredNorm()));
-        grad_norm = std::sqrt(comm.sum(F.eigen_vec().squaredNorm()));
+        grad_norm =
+            std::sqrt(comm.sum(eval_strain_field.eigen_vec().squaredNorm()));
 
         if ((verbose >= Verbosity::Detailed) and (comm.rank() == 0)) {
           std::cout << "at Newton step " << std::setw(count_width) << newt_iter
@@ -275,52 +361,51 @@ namespace muSpectre {
           using StrainMap_t = muGrid::FieldMap<Real, Mapping::Const>;
           if (verbose > Verbosity::Detailed) {
             std::cout << "<" << strain_symb << "> =" << std::endl
-                      << StrainMap_t{F, shape[0]}.mean() << std::endl;
+                      << StrainMap_t{eval_strain_field, shape[0]}.mean()
+                      << std::endl;
           }
         }
-        return convergence_test();
-      };
+        full_convergence_test();
+      }
 
-      if (cell.is_non_linear()) {
-        // Iterative update for the non-linear case
-        for (; newt_iter < solver.get_maxiter(); ++newt_iter) {
-          bool converged{iterate_solve()};
-          if (converged) {
-            break;
-          }
-        }
-        // throwing meaningful error message if the number of iterations for
-        // newton_cg is exploited
-        if (newt_iter == solver.get_maxiter()) {
-          std::stringstream err{};
-          err << "Failure at load step " << strain_step + 1 << " of "
-              << load_steps.size() << ". Newton-Raphson failed to converge. "
-              << "The applied boundary condition is Δ" << strain_symb << " ="
-              << std::endl
-              << macro_strain << std::endl
-              << "and the load increment is Δ" << strain_symb << " ="
-              << std::endl
-              << macro_strain - previous_macro_strain << std::endl;
-          throw ConvergenceError(err.str());
-        }
-      } else {
-        // Single strain update for the linear case
-        iterate_solve();
+      // throwing meaningful error message if the number of iterations for
+      // newton_cg is exploited
+      if (newt_iter == solver.get_maxiter()) {
+        std::stringstream err{};
+        err << "Failure at load step " << strain_step + 1 << " of "
+            << load_steps.size() << ". Newton-Raphson failed to converge. "
+            << "The applied boundary condition is Δ" << strain_symb << " ="
+            << std::endl
+            << macro_strain << std::endl
+            << "and the load increment is Δ" << strain_symb << " =" << std::endl
+            << macro_strain - previous_macro_strain << std::endl;
+        throw ConvergenceError(err.str());
       }
 
       // update previous macroscopic strain
       previous_macro_strain = macro_strain;
 
+      // re-evaluate cell in case the loop was terminated because of linearity
+      if (convergence_criterion.get_was_last_step_linear_test() and
+          not(convergence_criterion.get_newton_tol_test() or
+              convergence_criterion.get_equil_tol_test())) {
+        if (not(eigen_strain_func == muGrid::nullopt)) {
+          eval_strain_field = general_strain_field;
+          (eigen_strain_func.value())(strain_step, eval_strain_field);
+        }
+        cell.evaluate_stress_tangent();
+      }
+
       // store results
-      ret_val.emplace_back(
-          OptimizeResult{F.eigen_vec(), cell.get_stress().eigen_vec(),
-                         convergence_test(), Int(convergence_test()), message,
-                         newt_iter, solver.get_counter(), form});
+      ret_val.emplace_back(OptimizeResult{
+          general_strain_field.eigen_vec(), cell.get_stress().eigen_vec(),
+          full_convergence_test(), Int(full_convergence_test()), message,
+          newt_iter, solver.get_counter(), form});
 
       // store history variables for next load increment
       cell.save_history_variables();
+      convergence_criterion.reset();
     }
-
     return ret_val;
   }
 
@@ -329,28 +414,33 @@ namespace muSpectre {
   de_geus(Cell & cell, const LoadSteps_t & load_steps,
           KrylovSolverBase & solver, Real newton_tol, Real equil_tol,
           Verbosity verbose, IsStrainInitialised strain_init) {
+    if (load_steps.size() == 0) {
+      throw SolverError("No load steps specified.");
+    }
+
     const auto & comm = cell.get_communicator();
 
     using Matrix_t = Eigen::Matrix<Real, Eigen::Dynamic, Eigen::Dynamic>;
-
     auto shape{cell.get_strain_shape()};
 
+    ConvergenceCriterion convergence_criterion;
+
     // create a field collection to store workspaces
     auto field_collection{cell.get_fields().get_empty_clone()};
     // Corresponds to symbol δF or δε
     muGrid::MappedField<muGrid::FieldMap<Real, Mapping::Mut>> incrF_field{
-        "incrF",          shape[0], shape[1], IterUnit::SubPt,
-        field_collection, QuadPtTag};
+        "incrF",         shape[0],         shape[1],
+        IterUnit::SubPt, field_collection, QuadPtTag};
 
     // Corresponds to symbol ΔF or Δε
     muGrid::MappedField<muGrid::FieldMap<Real, Mapping::Mut>> DeltaF_field{
-        "DeltaF",           shape[0],         shape[1],
+        "DeltaF",        shape[0],         shape[1],
         IterUnit::SubPt, field_collection, QuadPtTag};
 
     // field to store the rhs for cg calculations
     muGrid::MappedField<muGrid::FieldMap<Real, Mapping::Mut>> rhs_field{
-        "rhs",    shape[0], shape[1], IterUnit::SubPt, field_collection,
-        QuadPtTag};
+        "rhs",           shape[0],         shape[1],
+        IterUnit::SubPt, field_collection, QuadPtTag};
     solver.initialise();
 
     size_t count_width{};
@@ -468,6 +558,7 @@ namespace muSpectre {
     // initialization of F (F in Formulation::finite_strain and ε in
     // Formuation::samll_strain)
     auto & F{cell.get_strain()};
+
     //! incremental loop
     for (const auto & tup : akantu::enumerate(load_steps)) {
       const auto & strain_step{std::get<0>(tup)};
@@ -484,22 +575,44 @@ namespace muSpectre {
       Real stress_norm{2 * equil_tol};
       bool has_converged{false};
 
-      auto convergence_test = [&incr_norm, &grad_norm, &newton_tol,
-                               &stress_norm, &equil_tol, &message,
-                               &has_converged]() {
-        bool incr_test = incr_norm / grad_norm <= newton_tol;
-        bool stress_test = stress_norm < equil_tol;
-        if (incr_test) {
-          message = "Residual  tolerance reached";
-        } else if (stress_test) {
-          message = "Reached stress divergence tolerance";
+      //! Checks two loop breaking criteria (newton tolerance and equilibrium
+      //! tolerance)
+      auto && early_convergence_test{
+          [&incr_norm, &grad_norm, &newton_tol, &stress_norm, &equil_tol,
+           &message, &has_converged, &convergence_criterion]() {
+            convergence_criterion.get_newton_tol_test() =
+                (incr_norm / grad_norm) <= newton_tol;
+            convergence_criterion.get_equil_tol_test() =
+                stress_norm < equil_tol;
+
+            if (convergence_criterion.get_newton_tol_test()) {
+              message = "Residual  tolerance reached";
+            } else if (convergence_criterion.get_equil_tol_test()) {
+              message = "Reached stress divergence tolerance";
+            }
+            has_converged = convergence_criterion.get_newton_tol_test() or
+                            convergence_criterion.get_equil_tol_test();
+            return has_converged;
+          }};
+
+      //! Checks all convergence criteria, including detection of linear
+      //! problems
+      auto && full_convergence_test{[&early_convergence_test, &cell,
+                                     &has_converged, &message,
+                                     &convergence_criterion]() {
+        convergence_criterion.get_was_last_step_linear_test() =
+            not cell.was_last_eval_non_linear();
+        if (convergence_criterion.get_was_last_step_linear_test()) {
+          message = "Linear problem, no more iteration necessary";
         }
-        has_converged = incr_test || stress_test;
+        has_converged = early_convergence_test() or
+                        convergence_criterion.get_was_last_step_linear_test();
         return has_converged;
-      };
+      }};
+
       Uint newt_iter{0};
 
-      if (cell.is_non_linear()) {
+      if (cell.was_last_eval_non_linear()) {
         // Iterative update in the nonlinear case
         for (; ((newt_iter < solver.get_maxiter()) and (!has_converged)) or
                (newt_iter < 2);
@@ -572,7 +685,7 @@ namespace muSpectre {
                         << StrainMap_t{F, shape[0]}.mean() << std::endl;
             }
           }
-          convergence_test();
+          full_convergence_test();
         }
         if (newt_iter == solver.get_maxiter()) {
           std::stringstream err{};
@@ -591,8 +704,10 @@ namespace muSpectre {
 
         // updating cell strain with the difference of the current and previous
         // strain input.
-        for (auto && strain : muGrid::FieldMap<Real, Mapping::Mut>(
-                 F, shape[0], muGrid::IterUnit::SubPt)) {
+        auto && F_map{muGrid::FieldMap<Real, Mapping::Mut>(
+            F, shape[0], muGrid::IterUnit::SubPt)};
+
+        for (auto && strain : F_map) {
           strain += macro_strain - previous_macro_strain;
         }
 
@@ -606,7 +721,7 @@ namespace muSpectre {
         cell.apply_projection(rhs);
         stress_norm = std::sqrt(comm.sum(rhs.eigen_vec().squaredNorm()));
 
-        if (!convergence_test()) {
+        if (!early_convergence_test()) {
           // calling sovler for solving the current (iteratively approximated)
           // linear equilibrium problem
           auto & incrF{incrF_field.get_field()};
@@ -648,22 +763,33 @@ namespace muSpectre {
                         << StrainMap_t{F, shape[0]}.mean() << std::endl;
             }
           }
+          full_convergence_test();
         }
       }
 
       // update previous macroscopic strain
       previous_macro_strain = macro_strain;
 
+      if (convergence_criterion.get_was_last_step_linear_test() and
+          not(convergence_criterion.get_newton_tol_test() or
+              convergence_criterion.get_equil_tol_test())) {
+        cell.evaluate_stress_tangent();
+      }
+
       // store results
       ret_val.emplace_back(
           OptimizeResult{F.eigen_vec(), cell.get_stress().eigen_vec(),
-                         convergence_test(), Int(convergence_test()), message,
-                         newt_iter, solver.get_counter(), form});
+                         full_convergence_test(), Int(full_convergence_test()),
+                         message, newt_iter, solver.get_counter(), form});
 
       // store history variables for next load increment
       cell.save_history_variables();
+      convergence_criterion.reset();
     }
 
     return ret_val;
   }
+
+  /* ---------------------------------------------------------------------- */
+
 }  // namespace muSpectre
